<?php

/*
 Plugin Name: Table of Contents Creator
 Plugin URI: http://markbeljaars.com/plugins/TOCC-plugin/
 Description: Table of Contents Creator (TOCC) automatically generates a dynamic site wide table of contents that is always up-to-date. All entries are navigable making your site very SEO friendly. TOCC can be configured to display static pages, blog entries and forum comments. Another great feature of TOCC is the ability to include anchor tags on any page marked with a special class. This feature allows links to articles, downloads or even other sites to appear within the table of contents as if they are part of your site's navigation. To generate a table of contents, simply include the <code>&lt;!--toc-creator--&gt;</code> tag on any page, or use the handy page creation feature located on the plugin admin page.
 Version: 1.3
 Author: Mark Beljaars
 Author URI: http://markbeljaars.com
 */

/*
 Copyright (C) 2009 Mark Beljaars, markbeljaars.com
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published 
 by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with this program.  If not, see http://www.gnu.org/licenses/.
 */

define('TOCC_VERSION', '1.3');
define('TOCC_PLUGIN_PATH', 'table-of-contents-creator');
define('TOCC_TAG', '<!-- toc-creator -->');
define('TOCC_CSS_FILE', 'tocc.css');
define('TOCC_HOLDER', '{x}');

$default_tocc_toc_title = '<h1>Table of Content</h1>';
$default_tocc_show_icons = '';
$default_tocc_date_format = 'F j, Y, H:i';
$default_tocc_show_ack = 'checked';
$default_tocc_show_static = 'checked';
$default_tocc_page_depth = 1;
$default_tocc_link_class = 'tocc';
$default_tocc_page_exclude = array ();
$default_tocc_show_blog = 'checked';
$default_tocc_blog_warn = 'checked';
$default_tocc_blog_warntext = '(last ' . constant('TOCC_HOLDER') . ' blogs shown)';
$default_tocc_blog_author = 'checked';
$default_tocc_blog_authorprefix = 'by:';
$default_tocc_blog_date = 'checked';
$default_tocc_blog_count = -1;
$default_tocc_blog_info = 'checked';
$default_tocc_blog_title = 'My Blog';
$default_tocc_blog_exclude = array ();
$default_tocc_show_forum = '';
$default_tocc_forum_page = 'Forum';
$default_tocc_forum_warn = 'checked';
$default_tocc_forum_warntext = '(last ' . constant('TOCC_HOLDER') . ' comments shown)';
$default_tocc_forum_author = 'checked';
$default_tocc_forum_authorprefix = 'by:';
$default_tocc_forum_date = 'checked';
$default_tocc_forum_count = 10;
$default_tocc_toc_page = 'Site Map';
$default_tocc_toc_slug = 'site-map';

add_option('tocc_version', '<1.2.1');
add_option('tocc_toc_title', $default_tocc_toc_title);
add_option('tocc_show_icons', $default_tocc_show_icons);
add_option('tocc_date_format', $default_tocc_date_format);
add_option('tocc_show_ack', $default_tocc_show_ack);
add_option('tocc_show_static', $default_tocc_show_static);
add_option('tocc_page_depth', $default_tocc_page_depth);
add_option('tocc_link_class', $default_tocc_link_class);
add_option('tocc_page_exclude', $default_tocc_page_exclude);
add_option('tocc_show_blog', $default_tocc_show_blog);
add_option('tocc_blog_warn', $default_tocc_blog_warn);
add_option('tocc_blog_warntext', $default_tocc_blog_warntext);
add_option('tocc_blog_author', $default_tocc_blog_author);
add_option('tocc_blog_authorprefix', $default_tocc_blog_authorprefix);
add_option('tocc_blog_date', $default_tocc_blog_date);
add_option('tocc_blog_count', $default_tocc_blog_count);
add_option('tocc_blog_info', $default_tocc_blog_info);
add_option('tocc_blog_title', $default_tocc_blog_title);
add_option('tocc_blog_exclude', $default_tocc_blog_exclude);
add_option('tocc_show_forum', $default_tocc_show_forum);
add_option('tocc_forum_page', $default_tocc_forum_page);
add_option('tocc_forum_warn', $default_tocc_forum_warn);
add_option('tocc_forum_warntext', $default_tocc_forum_warntext);
add_option('tocc_forum_author', $default_tocc_forum_author);
add_option('tocc_forum_authorprefix', $default_tocc_forum_authorprefix);
add_option('tocc_forum_date', $default_tocc_forum_date);
add_option('tocc_forum_count', $default_tocc_forum_count);
add_option('tocc_toc_page', $default_tocc_toc_page);
add_option('tocc_toc_slug', $default_tocc_toc_slug);

add_action('wp_head', 'tocc_head_hook');
add_action('admin_menu', 'tocc_menu_hook');
add_filter('the_content', 'tocc_display_hook');

tocc_check_upgrade();

function tocc_head_hook() {
	echo '<link rel="stylesheet" type="text/css" media="screen" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/' 
		. constant('TOCC_PLUGIN_PATH') . '/' . constant('TOCC_CSS_FILE') . '" />' . PHP_EOL;
}

function tocc_menu_hook() {
	add_options_page('Table of Contents Creator Options', 'Table of Contents Creator', 8, __FILE__, 'tocc_plugin_options');
}

function tocc_display_hook($content = '') {

	if (strpos($content, constant('TOCC_TAG')) === false)
		return $content;

	$tocc = PHP_EOL . PHP_EOL . '<!-- CREATED BY "TABLE OF CONTENT GENERATOR v' . constant('TOCC_VERSION') . '" BY MARK BELJAARS -->' 
			. PHP_EOL . PHP_EOL;

	if(get_option('tocc_toc_title')) {
		$tocc .= '<div id="tocc">' . get_option('tocc_toc_title') . PHP_EOL;
	}

	$pages = get_pages('depth=' . get_option('tocc_page_depth') . '&sort_column=menu_order');
	$lastpageID = "";
	$lastparentID = "";
	$depth = 0;

	foreach ($pages as $pagg) {

		if ($pagg->post_parent == 0) {
			$depth = 0;
			$lastparentID = $pagg->post_parent;
		}
		elseif ($pagg->post_parent == $lastpageID) {
			$depth++;
			$lastparentID = $pagg->post_parent;
		}
		elseif ($pagg->post_parent != $lastpageID && $pagg->post_parent != $lastparentID) {
			$depth--;
			$lastparentID = $pagg->post_parent;
		}

		if (get_option('tocc_show_static') && (!(get_option('tocc_page_exclude') && in_array($pagg->post_title
					, get_option('tocc_page_exclude')))) && (!get_option('tocc_show_blog') || $pagg->ID != get_option('page_for_posts'))
					&& (!get_option('tocc_show_forum') || $pagg->post_title != get_option('tocc_forum_page') 
					|| !function_exists('latest_activity'))) {
			$tocc .= tocc_display_static($pagg, $depth);
		}

		if (get_option('tocc_show_blog') && $pagg->ID == get_option('page_for_posts')) {
			$page_title = $pagg->post_title;
			if(!get_option('tocc_show_static')) {
				$page_title = get_option('tocc_blog_title');
			}
			$tocc .= tocc_display_posts($page_title, get_permalink($pagg->ID), $depth);
		}

		if (get_option('tocc_show_forum') && $pagg->post_title == get_option('tocc_forum_page') && function_exists('latest_activity')) {
			$tocc .= tocc_display_forum($pagg, $depth);
		}

		$lastpageID = $pagg->ID;
	}
	
	if (get_option('tocc_show_blog') && !get_option('page_for_posts')) {
		$tocc .= tocc_display_posts(get_option('tocc_blog_title'), get_option('home'));
	}

	if (get_option('tocc_show_ack')) {
		$tocc .= '<br \><br \>' . PHP_EOL . PHP_EOL
			   . '<div id="tocc_footer"><small>Generated by Table of Contents Creator v' . constant('TOCC_VERSION') . '<br \>' 
			   . 'by <a href="http://markbeljaars.com">Mark Beljaars</a></small></div>'. PHP_EOL;
   }

	$tocc .= '</div>';

	$content = str_replace(constant('TOCC_TAG'), $tocc, $content);
	return $content;
}

function tocc_check_upgrade() {

	if(get_option('tocc_version') != constant('TOCC_VERSION')) {

		$title = get_option('tocc_title');
		if($title && substr($title, 0, 1) != '<') {
			update_option('tocc_toc_title', '<h1>' . $title . '</h1>');
		}
	
		update_option('tocc_version',constant('TOCC_VERSION'));	
	}
}

function tocc_display_static($pagg = '', $depth = 0) {

	$content .= '<div id="tocc_depth' . $depth . '">' 
				. (get_option('tocc_show_icons') ? '<div id="tocc_page">' : '') 
				. '<a href="' . get_permalink($pagg->ID) . '">' . $pagg->post_title . '</a>' 
				. (get_option('tocc_show_icons') ? '</div>' : '')
				. '</div>' . PHP_EOL;

	if (get_option('tocc_link_class')) {
		preg_match_all('#<a\s+
							(?:(?= [^>]* href\s*=\s*["\']   (?P<href>  [^"\']*) ")|)
							(?:(?= [^>]* class\s*=\s*["\']  (?P<class> [^"\']*) ")|)
							[^>]*>
							(?P<text>[^<]*)
							</a>
							#xi', $pagg->post_content, $matches, PREG_SET_ORDER);

		foreach ($matches as $match) {
			if ($match['class'] == get_option('tocc_link_class')) {
				$content .= '<div id="tocc_depth' . ($depth +1) . '">' 
							. (get_option('tocc_show_icons') ? '<div id="tocc_page_link">' : '') 
							. '<a href="' . $match['href'] . '">' . $match['text'] . '</a>' 
							. (get_option('tocc_show_icons') ? '</div>' : '') 
							. '</div>' . PHP_EOL;
			}
		}
	}
	return $content;
}

function tocc_display_posts($pagename = '', $permalink = '', $depth = 0) {

	global $wpdb;

	if($pagename) {
		$content .= '<div id="tocc_depth' . $depth . '">' . (get_option('tocc_show_icons') ? '<div id="tocc_page">' : '') 
			. '<a href="' . $permalink . '">' . $pagename . '</a>' 
			. (get_option('tocc_blog_warn') && (int)get_option('tocc_blog_count') > 0 ? ' ' . str_replace(constant('TOCC_HOLDER')
				, get_option('tocc_blog_count'), get_option('tocc_blog_warntext')) : '')
			. (get_option('tocc_show_icons') ? '</div>' : '') . '</div>' . PHP_EOL;
		$depth++;
	}
	
	$categories = get_categories();
	foreach ($categories as $cat) {
		
		if (!(get_option('tocc_blog_exclude') && in_array($cat->cat_name, get_option('tocc_blog_exclude')))) {

			$usequery = true;
			$querystr = "
				SELECT $wpdb->posts.guid, $wpdb->posts.post_title, $wpdb->posts.post_date, $wpdb->posts.post_author
				FROM $wpdb->posts inner join $wpdb->term_relationships on $wpdb->term_relationships.object_id = $wpdb->posts.id 
					inner join $wpdb->terms on $wpdb->term_relationships.term_taxonomy_id = $wpdb->terms.term_id 
					inner join $wpdb->term_taxonomy on ($wpdb->term_taxonomy.taxonomy = 'category' and 
						$wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id)
				WHERE ($wpdb->posts.post_type like '%post%' AND post_status like '%publish%' AND wp_terms.term_id = " . $cat->cat_ID . ")
				ORDER BY post_date_gmt DESC"
				. (get_option('tocc_blog_count') > 0 ? " LIMIT 0," . get_option('tocc_blog_count') : "") . ";";

			$posts = $wpdb->get_results($querystr, OBJECT);
			if(!$posts) {
				$posts = get_posts('orderby=date&order=DESC&numberposts=' . get_option('tocc_blog_count') . '&cat=' . $cat->cat_ID);
				$usequery = false;
			}
			
			$content .= '<div id="tocc_depth' . ($depth) . '">' 
						. (get_option('tocc_show_icons') ? '<div id="tocc_blog_cat">' : '') 
						. '<a href="' . get_category_link($cat->cat_ID) . '">' . $cat->cat_name . '</a>' 
						. ($posts && get_option('tocc_blog_info')	? ' (' . $cat->count . ')' : '') 
						. (get_option('tocc_show_icons') ? '</div>' : '') 
						. '</div>' . PHP_EOL;

			if($posts) {
			
				$content .= '<div id="tocc_depth' . ($depth +1) . '">'
							. (get_option('tocc_show_icons') ? '<div id="tocc_blog_post">' : '') 
							. '<ul>';

				foreach ($posts as $post) {
					$user = get_userdata($post->post_author);
					$content .= '<li>' 
								. '<a href="' . ( $usequery ? $post->guid : get_permalink($post->ID)) . '">' . $post->post_title . '</a> '
								. (get_option('tocc_blog_author') ?  get_option('tocc_blog_authorprefix') . ' <a href="' 
									. $user->user_url . '">' . $user->display_name . '</a>' : '') 
								. (get_option('tocc_blog_date') ? '<br /><small>' . date(get_option('tocc_date_format')
									, strtotime($post->post_date)) . '</small>' : '') 
								. '</li>' . PHP_EOL;
				}
				$content .= '</ul>' 
							. (get_option('tocc_show_icons') ? '</div>' : '') 
							. '</div>' . PHP_EOL;

			}
		}
	}
	return $content;
}

function tocc_display_forum($pagg = '', $depth = 0) {
	
	global $wpdb;

	ob_start();
	latest_activity((int)get_option('tocc_forum_count') > 0 ? (int)get_option('tocc_forum_count') : 100000);
	$posts = ob_get_contents();
	ob_end_clean();
	
	$content .= '<div id="tocc_depth' . $depth . '">' 
				. (get_option('tocc_show_icons') ? '<div id="tocc_page">' : '') 
				. '<a href="' . get_permalink($pagg->ID) . '">' . $pagg->post_title . '</a>'
				. (get_option('tocc_forum_warn') && (int)get_option('tocc_forum_count') > 0 ? ' ' . str_replace(constant('TOCC_HOLDER')
					, get_option('tocc_forum_count'), get_option('tocc_forum_warntext')) : '')
				. (get_option('tocc_show_icons') ? '</div>' : '') 
				. '</div>' . PHP_EOL
				. '<div id="tocc_depth' . ($depth +1) . '">' . (get_option('tocc_show_icons') ? '<div id="tocc_forum">' : '')
				. '<ul>';

	preg_match_all('#<li>(.*?)</li>#xi', $posts, $matches, PREG_SET_ORDER);
	foreach ($matches as $match) {
		
		preg_match_all('#(.*?)</a>#xi', $match[1], $comment, PREG_SET_ORDER);
		preg_match_all('#</a>\s*by:\s*(.*?)<br\s*/>#xi',$match[1], $author, PREG_SET_ORDER);
		preg_match_all('#<small>(.*?)</small>#xi', $match[1], $date, PREG_SET_ORDER);
		
		$content .= '<li>'
					. $comment[0][0]
					. (get_option('tocc_forum_author') ? ' ' . get_option('tocc_forum_authorprefix') . ' ' . $author[0][1] : '')
					. (get_option('tocc_forum_date') ? '<br /><small>' .  date(get_option('tocc_date_format'), strtotime($date[0][1])) . '</small>' : '')
					. '</li>';
	}

	$content .= '</ul>' 
				. (get_option('tocc_show_icons') ? '</div>' : '') 
				. '</div>' . PHP_EOL;
				
	return $content;
}

function tocc_option($option_text, $tooltip_text, $option_name, $element_type, $other_option = '', $disabled = 0, 
			$element_index = 'post_title') {

	echo '<tr><th scope="row" style="text-align:right; vertical-align:top;"><a style="cursor:pointer;" title="Click for Help!"'
		. 'onclick="toggleVisibility(\'' . $option_name . '_tip\');">' . ($disabled ? '<span style="color: grey">' : '') 
		. $option_text . ':' . ($disabled ? '</span>' : '') . '</a></th><td align="left" valign="middle">';

	switch ($element_type) {
		case 'text' :
			echo '<input name="' . $option_name . '" type="text" size="' . $other_option . '" value="' 
				. htmlspecialchars(get_option($option_name)) . '"' . ($disabled ? ' disabled="true"' : '') . ' />';
			break;
		case 'checkbox' :
			if (!is_array($other_option)) {
				echo '<input type="checkbox" name="' . $option_name . '"' . (get_option($option_name) ? ' checked="checked"' : '') 
					. ($disabled ? ' disabled="true"' : '') . ' />';
			} else {
				foreach ($other_option as $opts) {
					echo '<input type="checkbox" name="' . $option_name . '[]" value="' . $opts-> {$element_index}
					 . '"' . (get_option($option_name) && in_array($opts-> {$element_index}, get_option($option_name)) 
					 ? ' checked="checked"' : '') . ($disabled ? ' disabled="true"' : '') . '> ' 
					 . ($disabled ? '<span style="color: grey">' : '') . $opts-> {$element_index}
					 . ($disabled ? '</span>' : '') . '<br /></input>';
				}
			}
			break;
		case 'select' :
			echo '<select name="' . $option_name . '"' . ($disabled ? ' disabled="true"' : '') . '>';
			foreach ($other_option as $opts) {
				echo '<option value="' . $opts-> {$element_index} . '"' 
					. (htmlspecialchars(get_option($option_name)) == $opts-> {$element_index} ? ' selected="selected"' : '') 
					. '>' . $opts-> {$element_index} . '</option>';
			}
			echo '</select>';
			break;
	}

	echo '<div style="max-width:500px; text-align:left; display:none" id="' . $option_name . '_tip"">' . $tooltip_text 
		. '</div></td></tr>';
}

function tocc_plugin_options() {

	global $default_tocc_toc_title;
	global $default_tocc_show_icons;
	global $default_tocc_date_format;
	global $default_tocc_show_ack;
	global $default_tocc_show_static;
	global $default_tocc_page_depth;
	global $default_tocc_link_class;
	global $default_tocc_page_exclude;
	global $default_tocc_show_blog;
	global $default_tocc_blog_warn;
	global $default_tocc_blog_warntext;
	global $default_tocc_blog_author;
	global $default_tocc_blog_authorprefix;
	global $default_tocc_blog_date;
	global $default_tocc_blog_count;
	global $default_tocc_blog_info;
	global $default_tocc_blog_title;
	global $default_tocc_blog_exclude;
	global $default_tocc_show_forum;
	global $default_tocc_forum_page;
	global $default_tocc_forum_warn;
	global $default_tocc_forum_warntext;
	global $default_tocc_forum_author;
	global $default_tocc_forum_authorprefix;
	global $default_tocc_forum_date;
	global $default_tocc_forum_count;
	global $default_tocc_toc_page;
	global $default_tocc_toc_slug;

	echo '<script type="text/javascript">' . PHP_EOL 
		. '<!--' . PHP_EOL 
		. 'function toggleVisibility(id) { ' . PHP_EOL 
		. '   var e = document.getElementById(id);' . PHP_EOL 
		. '   if(e.style.display == "block")' . PHP_EOL 
		. '      e.style.display = "none";' . PHP_EOL 
		. '   else' . PHP_EOL 
		. '      e.style.display = "block";' . PHP_EOL 
		. '}' . PHP_EOL 
		. '//-->' . PHP_EOL 
		. '</script>' . PHP_EOL 
		. PHP_EOL;

	echo '<div class="wrap"><h2>Table of Content Creator v' . constant('TOCC_VERSION') . '</h2>' . PHP_EOL 
		. '<p>The TOC Creator Plugin for Wordpress automatically generates a site wide table of content for your blog. The TOC will ' 
		. 'always remain up date and is a SEO freindly way of linking all pages on your site.</p>' . PHP_EOL 
		. '<p>To use the plugin, insert the trigger text <strong>' . htmlspecialchars(constant('TOCC_TAG')) 
		. '</strong> into an existing page. The trigger will be auomatically replaced with a complete site wide table of content. ' 
		. 'You may also use the <code>create page</code> button below.</p>' . PHP_EOL 
		. '<h3>Click on option titles to get help!</h3>' . PHP_EOL;

	if (isset ($_POST['set_defaults'])) {

		update_option('tocc_toc_title', $default_tocc_toc_title);
		update_option('tocc_show_icons', $default_tocc_show_icons);
		update_option('tocc_date_format', $default_tocc_date_format);
		update_option('tocc_show_ack', $default_tocc_show_ack);
		update_option('tocc_show_static', $default_tocc_show_static);
		update_option('tocc_page_depth', $default_tocc_page_depth);
		update_option('tocc_link_class', $default_tocc_link_class);
		update_option('tocc_page_exclude', $default_tocc_page_exclude);
		update_option('tocc_show_blog', $default_tocc_show_blog);
		update_option('tocc_blog_info', $default_tocc_blog_info);
		update_option('tocc_blog_count', $default_tocc_blog_count);
		update_option('tocc_blog_title', $default_tocc_blog_title);
		update_option('tocc_blog_warn', $default_tocc_blog_warn);
		update_option('tocc_blog_warntext', $default_tocc_blog_warntext);
		update_option('tocc_blog_author', $default_tocc_blog_author);
		update_option('tocc_blog_authorprefix', $default_tocc_blog_authorprefix);
		update_option('tocc_blog_date', $default_tocc_blog_date);
		update_option('tocc_blog_exclude', $default_tocc_blog_exclude);
		update_option('tocc_show_forum', $default_tocc_show_forum);
		update_option('tocc_forum_page', $default_tocc_forum_page);
		update_option('tocc_forum_warn', $default_tocc_forum_warn);
		update_option('tocc_forum_warntext', $default_tocc_forum_warntext);
		update_option('tocc_forum_author', $default_tocc_forum_author);
		update_option('tocc_forum_authorprefix', $default_tocc_forum_authorprefix);
		update_option('tocc_forum_date', $default_tocc_forum_date);
		update_option('tocc_forum_count', $default_tocc_forum_count);
		update_option('tocc_toc_page', $default_tocc_toc_page);
		update_option('tocc_toc_slug', $default_tocc_toc_slug);

		echo '<div id="message" class="updated fade"><p><strong>Default options loaded!</strong></p></div>';

	} else
		if (isset ($_POST['create_page'])) {

			$title = stripslashes((string) $_POST['tocc_toc_page']);
			$slug = stripslashes((string) $_POST['tocc_toc_slug']);

			update_option('tocc_toc_page', $title);
			update_option('tocc_toc_slug', $slug);

			$postID = wp_insert_post(array (
				'post_title' => $title,
				'post_content' => constant('TOCC_TAG'),
				'post_status' => 'publish',
				'post_author' => 1,
				'post_name' => $slug,
				'post_type' => 'page'
			));

			echo '<div id="message" class="updated fade"><p><strong>TOC page ' . ($postID ? '(ID ' . $postID . ') was' : 'could not be') 
				. ' created.</strong></p></div>';

		} else
			if (isset ($_POST['info_update'])) {

				update_option('tocc_toc_title', stripslashes((string) $_POST['tocc_toc_title']));
				update_option('tocc_show_icons', stripslashes((string) $_POST['tocc_show_icons']));
				update_option('tocc_date_format', (string) $_POST['tocc_date_format']);
				update_option('tocc_show_ack', stripslashes((string) $_POST['tocc_show_ack']));
				update_option('tocc_show_static', stripslashes((string) $_POST['tocc_show_static']));
				update_option('tocc_page_depth', intval(stripslashes((string) $_POST['tocc_page_depth'])));
				update_option('tocc_link_class', stripslashes((string) $_POST['tocc_link_class']));
				update_option('tocc_page_exclude', $_POST['tocc_page_exclude']);
				update_option('tocc_show_blog', stripslashes((string) $_POST['tocc_show_blog']));
				update_option('tocc_blog_info', stripslashes((string) $_POST['tocc_blog_info']));
				update_option('tocc_blog_warn', stripslashes((string) $_POST['tocc_blog_warn']));
				update_option('tocc_blog_warntext', stripslashes((string) $_POST['tocc_blog_warntext']));
				update_option('tocc_blog_author', stripslashes((string) $_POST['tocc_blog_author']));
				update_option('tocc_blog_authorprefix', stripslashes((string) $_POST['tocc_blog_authorprefix']));
				update_option('tocc_blog_date', stripslashes((string) $_POST['tocc_blog_date']));
				update_option('tocc_blog_count', intval(stripslashes((string) $_POST['tocc_blog_count'])));
				update_option('tocc_blog_title', stripslashes((string) $_POST['tocc_blog_title']));
				update_option('tocc_blog_exclude', $_POST['tocc_blog_exclude']);
				update_option('tocc_show_forum', stripslashes((string) $_POST['tocc_show_forum']));
				update_option('tocc_forum_page', stripslashes((string) $_POST['tocc_forum_page']));
				update_option('tocc_forum_warn', stripslashes((string) $_POST['tocc_forum_warn']));
				update_option('tocc_forum_warntext', stripslashes((string) $_POST['tocc_forum_warntext']));
				update_option('tocc_forum_author', stripslashes((string) $_POST['tocc_forum_author']));
				update_option('tocc_forum_authorprefix', stripslashes((string) $_POST['tocc_forum_authorprefix']));
				update_option('tocc_forum_date', stripslashes((string) $_POST['tocc_forum_date']));
				update_option('tocc_forum_count', intval(stripslashes((string) $_POST['tocc_forum_count'])));
				update_option('tocc_toc_page', stripslashes((string) $_POST['tocc_toc_page']));
				update_option('tocc_toc_slug', stripslashes((string) $_POST['tocc_toc_slug']));

				echo '<div id="message" class="updated fade"><p><strong>Configuration updated!</strong></p>';
				if (!get_option('tocc_show_ack')) {
					echo '<br />The author of this software has spent uncountable hours of his own time to develop and test this '
					     . 'plugin only to offer it free to you. Please consider leaving the author acknowledgment enabled as a small '
					     . 'token of your appreciation.';
				}
				
				echo '</div>';
			}

	echo PHP_EOL . '<form method="post" action="" name="dofollow"><table class="form-table">' . PHP_EOL;

	echo PHP_EOL . '<tr><td><h3>General</h3></td><td>&nbsp;</td></tr>';
	tocc_option('TOC Title', 'This text entered here will be displayed at the top of the table of content page. Leave blank to display ' 
		. 'no title. Include any required formatting tags.', 'tocc_toc_title', 'text', '40');
	tocc_option('Show Icons', 'Displays a small icon next to each table of contents entry. Icons are used to differentiate pages, ' 
		. 'links, blog categories, blog entries and form comments.', 'tocc_show_icons', 'checkbox');
	tocc_option('Date Format', 'Used to set the date display format when showing blog or forum comment dates. See '
		. '<a href="http://php.net/manual/en/function.date.php">http://php.net/manual/en/function.date.php</a> for format string details.'
		, 'tocc_date_format', 'text', '40');
	tocc_option('Show Author Credit', 'Displays a small unobtrusive acknowledgement at the bottom of the table of contents. ' 
		. 'Please consider leaving the acknowledgement enabled as a token of appreciation for the hours spent developing this plugin.'
		, 'tocc_show_ack', 'checkbox');

	echo PHP_EOL . '<tr><td colspan="2"><h3>Static Pages</h3></td></tr>';
	tocc_option('Show Static Pages', 'Show static pages within the table of contents. Note that the pages will be sorted in menu order. ' 
		. 'Use the <strong>order</strong> attribute of each page (settable via page edit) to set the menu order.', 'tocc_show_static'
		, 'checkbox');
	tocc_option('Page Depth', 'Determines the page depth to display. For example if set to 1, all parent pages will be shown and the ' 
		. 'immediate children of each parent page will also be shown. Set to 0 to show parent pages only.', 'tocc_page_depth', 'text', '5');
	tocc_option('Link Class', 'If set, any &lt;a&gt; tags assigned to this class will be shown as a list item under the page in which the ' 
		. 'link was found. To use, add class="<em>linkclass</em>" to each &lt;a&gt; tag you wish to display in the table of ' 
		. 'contents. Leave blank to disable this option.', 'tocc_link_class', 'text', '40');
	tocc_option('Exclude Pages', 'Select one or more pages to exclude from the table of content. By default, all pages are enabled.'
		, 'tocc_page_exclude', 'checkbox', get_pages('depth=-1&sort_column=menu_order'));

	$no_blog = !is_blog_installed();
	echo PHP_EOL . '<tr><td colspan="2"><h3>Blog Entries</h3></td></tr>';
	tocc_option('Show Blog Entries', 'Show blog entries within the table of contents. Blog entries will automatically be separated by ' 
		. 'category.', 'tocc_show_blog', 'checkbox', '', $no_blog);
	tocc_option('Blog Title', 'This is a mock page name used for sites that do not have a static post page. See the Wordpress ' 
		. 'Settings/Reading option for more details. This blog title is also used if static pages are not displayed. This field may '
		. 'may be left blank.', 'tocc_blog_title', 'text', '40', $no_blog);
	tocc_option('Show Posts Total', 'Displays the total number of posts in brackets for each category next to each category listing.'
		, 'tocc_blog_info', 'checkbox', '', $no_blog);
	tocc_option('Number of Blog Entries', 'Determines the maximum number of blog entries to display in each category. Set to 0 to '
		. 'display all posts.', 'tocc_blog_count', 'text', '5', $no_blog);
	tocc_option('Show Posts Warning', 'Displays a message after the blog page name to alert the reader that only the latest <em>n</em> '
		. 'posts are shown. This text is not shown if all posts are displayed (by entering 0 in the "Number of Blog Entries" setting '
		. 'below).', 'tocc_blog_warn', 'checkbox', '', $no_blog);
	tocc_option('Posts Warning Text', 'The message to display if the post warning is enabled. Use <code>' . constant('TOCC_HOLDER') 
		. '</code> as a place holder for the number of blog entries being displayed.', 'tocc_blog_warntext', 'text', '40', $no_blog);
	tocc_option('Show Post Author', 'Displays the author of the post.', 'tocc_blog_author', 'checkbox', '', $no_blog);
	tocc_option('Post Author Prefix', 'The text to be displayed before the authors name.', 'tocc_blog_authorprefix', 'text', '5'
		, $no_blog);
	tocc_option('Show Post Date', 'Displays the creation date of the post.', 'tocc_blog_date', 'checkbox', '', $no_blog);
	tocc_option('Exclude Categories', 'Select one or more categories to exclude from the table of content. By default, all categories ' 
		. 'are enabled.', 'tocc_blog_exclude', 'checkbox', $categories = get_categories(), $no_blog, 'cat_name');

	$no_forum = !function_exists('latest_activity');
	echo PHP_EOL . '<tr><td colspan="2"><h3>Forum</h3>Note that currently this option only works with <em>Forum Server</em> and ' 
		. '<em>WP-Forum</em>.</td></tr>' . PHP_EOL;
	tocc_option('Show Forum Comments', 'Show forum comments within the table of contents. The latest forum comments will be shown first. ' 
		. 'Note that currently this option only works with <em>Forum Server</em> and <em>WP-Forum</em>.', 'tocc_show_forum', 'checkbox', ''
		, $no_forum);
	tocc_option('Forum Page Name', 'This is the name of the WordPress page that is used as a place holder to display the forum. The page ' 
		. 'name is required so that the plugin can display the forum comments underneath the correct page name. The forum page ' 
		. 'typically consists only of a tag such as &lt;!--VASTHTML--&gt;.', 'tocc_forum_page', 'select'
		, get_pages('depth=-1&sort_column=menu_order'), $no_forum);
	tocc_option('Number of Forum Comments', 'Determines the maximum number of forum comments to display. The latest comments will be ' 
		. 'displayed first. Set to 0 or less to display all comments (not recommended).', 'tocc_forum_count', 'text', '5', $no_forum);
	tocc_option('Show Comments Warning', 'Displays a message after the forum page name to alert the reader that only the latest <em>n</em> '
		. 'comments are shown. This text is not shown if all comments are displayed (by entering 0 in the "Number of Forum Comments" '
		. 'setting below).', 'tocc_forum_warn', 'checkbox', '', $no_forum);
	tocc_option('Comment Warning Text', 'The message to display if the comment warning is enabled. Use <code>' . constant('TOCC_HOLDER') 
		. '</code> as a place holder for the number of comment entries being displayed.', 'tocc_forum_warntext', 'text', '40', $no_forum);
	tocc_option('Show Comment Author', 'Displays the author of the comment.', 'tocc_forum_author', 'checkbox', '', $no_forum);
	tocc_option('Comment Author Prefix', 'The text to be displayed before the authors name.', 'tocc_forum_authorprefix', 'text', '5'
		, $no_forum);
	tocc_option('Show Comment Date', 'Displays the creation date of the forum comment.', 'tocc_forum_date', 'checkbox', '', $no_forum);

	echo PHP_EOL . '<tr><td colspan="2"><h3>Update</h3>In order to save any changes, you must press the <em>Update Options</em> button' 
		. 'before leaving this page or using the create page facility below.</td></tr>' . PHP_EOL;
	echo '</table>' . PHP_EOL
		. '<div class="submit">'
		. '<input type="submit" name="set_defaults" value="Load Default Options &raquo;" />&nbsp;&nbsp'
		. '<input type="submit" name="info_update" value="Update Options &raquo;" />'
		. '</div>' . PHP_EOL;

	echo PHP_EOL . '<table class="form-table">' . PHP_EOL
		. '<tr><td colspan="2"><h3>Create Page</h3>For your convenience, the plugin can also create a new table of content page for you. ' 
		. 'Simply fill in the title and slug (path) details and press the <em>Create Page</em> button to create the page. The trigger text ' 
		. 'will be added automatically to the new page.</td></tr>' . PHP_EOL
		. '<tr><td colspan="2"><strong>Please note: </strong>If you have changed any of the default settings, make sure you have pressed '
		. 'the <em>Update Options</em> button above before creating the new page.</td></tr>' . PHP_EOL;
	tocc_option('Page Title', 'The table of content page name. The newly created page will be called this name. Ensure that there are no ' 
		. 'current pages of the same name.', 'tocc_toc_page', 'text', '40');
	tocc_option('Page Slug', 'The page slug is used to determine the address of the page. It is typically the same name as the page name, ' 
		. '<strong>however</strong> the page slug cannot contain spaces. It is common practice to replace a space with a dash.'
		, 'tocc_toc_slug', 'text', '40');

	echo PHP_EOL . '</table>' . PHP_EOL
		. '<div class="submit">'
		. '<input type="submit" name="create_page" value="Create Page &raquo" />'
		. '</div></form></div>' . PHP_EOL;

}
?>
